package fred;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.util.Vector;
/**
 * <p>Class setting the apropriate position of the window on desktop. It keeping all windows fully
 * visible inside of the desktop.</p>
 * @author Damian Jarzbski
 */
public class MyDesktopLayout implements LayoutManager2, java.io.Serializable {
	Vector VComp;

	public MyDesktopLayout() {
		VComp = new Vector();
	}

	public void addLayoutComponent(Component comp) {
		synchronized (comp.getTreeLock()) {
			VComp.add(comp);
		}
	}

	public void removeLayoutComponent(Component comp) {
		synchronized (comp.getTreeLock()) {
			VComp.remove(comp);
		}
	}

	public Dimension minimumLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			Dimension dim = new Dimension(0, 0);

			for (int i = 0; i < VComp.size(); i++) {
				dim.height = Math.max(dim.height, ((Component)VComp.elementAt(i)).getHeight());
				dim.width = Math.max(dim.width, ((Component)VComp.elementAt(i)).getWidth());
			}

			return dim;
		}
	}

	public Dimension preferredLayoutSize(Container target) {
		synchronized (target.getTreeLock()) {
			Dimension dim = new Dimension(0, 0);

			Component c;
			if ((c = (Component)VComp.elementAt(0)) != null) {
				Dimension d = c.getPreferredSize();
				dim.width += d.width;
				dim.height = Math.max(d.height, dim.height);
			}

			Insets insets = target.getInsets();
			dim.width += insets.left + insets.right;
			dim.height += insets.top + insets.bottom;

			return dim;
		}
	}

	/**
	 * Returns the maximum dimensions for this layout given the components
	 * in the specified target container.
	 * @param target the component which needs to be laid out
	 * @see Container
	 * @see #minimumLayoutSize
	 * @see #preferredLayoutSize
	 */
	public Dimension maximumLayoutSize(Container target) {
		return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
	}

	/**
	 * Returns the alignment along the x axis.  This specifies how
	 * the component would like to be aligned relative to other
	 * components.  The value should be a number between 0 and 1
	 * where 0 represents alignment along the origin, 1 is aligned
	 * the furthest away from the origin, 0.5 is centered, etc.
	 */
	public float getLayoutAlignmentX(Container parent) {
		return 0.5f;
	}

	/**
	 * Returns the alignment along the y axis.  This specifies how
	 * the component would like to be aligned relative to other
	 * components.  The value should be a number between 0 and 1
	 * where 0 represents alignment along the origin, 1 is aligned
	 * the furthest away from the origin, 0.5 is centered, etc.
	 */
	public float getLayoutAlignmentY(Container parent) {
		return 0.5f;
	}

	/**
	 * Invalidates the layout, indicating that if the layout manager
	 * has cached information it should be discarded.
	 */
	public void invalidateLayout(Container target) {}

	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			Insets insets = target.getInsets();
			int top = insets.top;
			int bottom = target.getHeight() - insets.bottom;
			int left = insets.left;
			int right = target.getWidth() - insets.right;

			Component c = null;
			Insets ins = new Insets(0, 0, 0, 0);

			for (int i = 0; i < VComp.size(); i++) {
				c = (Component)VComp.elementAt(i);
				ins.left = c.getLocation().x;
				ins.top = c.getLocation().y;
				ins.right = c.getWidth();
				ins.bottom = c.getHeight();
				if (ins.left < left)
					ins.left += left - ins.left;
				if ((ins.left + ins.right) > right)
					if ((right - ins.right) < left) {
						ins.right = right-left;
					}
					else
						ins.left -= ((ins.left + ins.right) - right);
				if ((ins.top + ins.bottom) > bottom)
					ins.top -= ((ins.top + ins.bottom) - bottom);
				if (ins.top < top)
					if (top>bottom-ins.bottom) {
						ins.top += top - ins.top;
						ins.bottom=bottom-top;
					}
					else
						ins.top += top - ins.top;
				Dimension d = c.getMinimumSize();
				c.setBounds(ins.left, ins.top, (d.width<ins.right)?ins.right:d.width, (d.height<ins.bottom)?ins.bottom:d.height);
			}
		}
	}

	/**
	 * Returns a string representation of the state of this border layout.
	 * @return    a string representation of this border layout.
	 */
	public String toString() {
		return getClass().getName();
	}

	public void addLayoutComponent(Component comp, Object constraints) {
		addLayoutComponent(comp);
	}

	public void addLayoutComponent(String name, Component comp) {
		addLayoutComponent(comp);
	}
}
